#include "nSysimp.h"
#include <nSystem.h>
#include <stdio.h>

int nMulticast(nTask *parm_listeners, int n, void *msg, int dt) {
  int i, rc;
  nTask *listeners= (nTask*)nMalloc(n*sizeof(nTask));
  START_CRITICAL();
  current_task->listen_count= 0;
  current_task->reply_count= 0;
  current_task->listeners= listeners;
  current_task->size_listeners= n;
  current_task->msg= msg;

  for (i=0; i<n; i++) {
    listeners[i]= parm_listeners[i];
    if (listeners[i]->status==WAIT_MCAST) {
      listeners[i]->status= READY;
      PushTask(ready_queue, listeners[i]);
      listeners[i]= NULL;
      current_task->listen_count++;
    }
  }

  if(current_task->listen_count==n)
    current_task->status= WAIT_REPLY_MCAST;
  else if (dt>0) {
    current_task->status= WAIT_LISTEN_TIMEOUT;
    ProgramTask(dt);
  }
  else
    current_task->status= WAIT_LISTEN;

  ResumeNextReadyTask();

  if (current_task->listen_count!=current_task->reply_count) {
    current_task->status= WAIT_REPLY_MCAST;
    ResumeNextReadyTask();
  }

  nFree(listeners);
  rc= current_task->listen_count;
  END_CRITICAL();
  return rc;
}

void *nListen(nTask sender) {
  void *msg;
  START_CRITICAL();

  if (sender->status!=WAIT_LISTEN && sender->status!=WAIT_LISTEN_TIMEOUT) {
    current_task->status= WAIT_MCAST;
    ResumeNextReadyTask();
  }
  else {
    int i;
    for (i= 0; i<sender->size_listeners; i++)
      if (sender->listeners[i]==current_task) {
        sender->listen_count++;
        if (sender->listen_count==sender->size_listeners) {
          if (sender->status==WAIT_LISTEN_TIMEOUT)
            CancelTask(sender);
          sender->status= WAIT_REPLY_MCAST;
        }
        sender->listeners[i]= NULL;
        break;
      }
    if (i>=sender->size_listeners) {
      current_task->status= WAIT_MCAST;
      ResumeNextReadyTask();
    }
  }

  msg= sender->msg;
  END_CRITICAL();
  return msg;
}

void nReplyMulticast(nTask sender) {
  START_CRITICAL();
  sender->reply_count++;
  if (sender->reply_count==sender->listen_count &&
      sender->status==WAIT_REPLY_MCAST) {
    sender->status= READY;
    PushTask(ready_queue, current_task);
    PushTask(ready_queue, sender);
    ResumeNextReadyTask();
  } 

  END_CRITICAL();
}
